#if !defined AUTO_VECTOR
#define AUTO_VECTOR
//------------------------------------
//  auto_vector
//  (c) Bartosz Milewski, 1998
//------------------------------------
#include <memory>
#include <cassert>
using std::auto_ptr;

template<class T>   
class auto_vector;

template<class T>
class const_auto_iterator: public
	std::iterator<std::forward_iterator_tag, T const *>
{
    friend class auto_vector<T>;
public:
	const_auto_iterator () : _pp (NULL) {}
	const_auto_iterator (auto_ptr<T> const * pp) : _pp (pp) {}
    const_auto_iterator (const_auto_iterator<T> const & it) : _pp(it._pp) {}

	bool operator != (const_auto_iterator<T> const & it) const 
		{ return it._pp != _pp; }
	const_auto_iterator operator++ (int) { return _pp++; }
	const_auto_iterator operator++ () { return ++_pp; }
	T const * operator * () { return _pp->get (); }
	T const * operator-> () { return _pp->get (); }

private:
    auto_ptr<T> const * _memoryaddress() { return _pp;}
	auto_ptr<T> const * _pp;
};

template<class T>
class auto_iterator: public
	std::iterator<std::forward_iterator_tag, T *>
{
    friend class auto_vector<T>;
public:
	auto_iterator () : _pp (NULL) {}
	auto_iterator (auto_ptr<T> * pp) : _pp (pp) {}
    auto_iterator (auto_iterator const & it) : _pp (it._pp) {}

	bool operator != (auto_iterator<T> const & it) const 
		{ return it._pp != _pp; }

	auto_iterator operator++ (int) { return _pp++; }
	auto_iterator operator++ () { return ++_pp; }
	T * operator * () { return _pp->get (); }
	T * operator-> () { return _pp->get (); }

protected:
    auto_ptr<T> * _memoryaddress() { return _pp;}

private:
    auto_ptr<T> * _pp;
};



template <class T> 
class auto_vector
{
public:
	explicit auto_vector (size_t capacity = 0)
	{
        _arr = (capacity != 0) ? new auto_ptr<T> [capacity] : NULL;
        _end = 0;
        _capacity = capacity;
        if(_arr != NULL)
        {
            assert(_capacity != 0);
        }
	}

    auto_vector(auto_vector &av)
    {
        (*this) = av;
    }

	~auto_vector ()
	{ 
        empty();
    }

    auto_vector & operator = (auto_vector &av)
    {
        if(size() > 0)
            empty();

        _arr = av._arr;
        _capacity = av._capacity;
        _end = av._end;

        av._arr = NULL;
        av._capacity = 0;
        av._end = 0;

        return (*this);
    }


	size_t size () const { return _end; }
	T const * operator [] (size_t i) const
	{
		assert (i < _end);
		return _arr [i].get ();
	}
	T * operator [] (size_t i)
	{
		assert (i < _end);
		return _arr [i].get ();
	}
	void assign (size_t i, auto_ptr<T> & p)
	{
		assert (i < _end);
		_arr [i] = p;
	}
	void assign_direct (size_t i, T * p)
	{
		assert (i < _end);
		_arr [i].reset (ptr);
	}
	void push_back (auto_ptr<T> & p)
	{
		if (_end == _capacity)
			grow (_end + 1);
		_arr [_end++] = p;
	}
	auto_ptr<T> pop_back ()
	{
		assert (_end != 0);
		return _arr [--_end];
	}

	typedef const_auto_iterator<T> const_iterator;
	typedef auto_iterator<T> iterator;

	iterator begin() { return _arr;}
	iterator end() { return _arr+_end;}

	const_iterator const_begin () const { return _arr; }
	const_iterator const_end () const { return _arr + _end; }

    void remove(iterator &it)
	{    
        assert(it._pp >= _arr && it._pp <= (_arr+_end));

        if(it != end())
        {
            assert(_end != 0);

            iterator tempit(it);
            iterator preit(tempit);
            iterator curit(++tempit);
            preit._pp->reset(NULL);
            assert(!(*preit));
            
            while(curit != end())
            {
                *((preit++)._pp) = *((curit++)._pp);
            }

            --_end;
        }
    }

	iterator find(T *p)
	{
		iterator it;
		for(it = begin(); it != end(); ++it)
		{
			if(*it == p)
				break;
		}
        return it;
	}

	void empty()
	{
        if(_arr != 0)
        {
            assert(_capacity != 0);
            delete [] _arr;
        }
		_arr = 0;
		_end = 0;
		_capacity = 0;
	}

private:	
	void grow (size_t reqCapacity);

    int someshit;
	auto_ptr<T>  *_arr;
	size_t		  _capacity;
	size_t		  _end;
};

template <class T>
void auto_vector<T>::grow (size_t reqCapacity)
{
	size_t newCapacity = 2 * _capacity;
	if (reqCapacity > newCapacity)
		newCapacity = reqCapacity;
	// allocate new array
	auto_ptr<T> * arrNew = new auto_ptr<T> [newCapacity];
	// transfer all entries
	for (size_t i = 0; i < _capacity; ++i)
		arrNew [i] = _arr [i];
	_capacity = newCapacity;
	// free old memory
    if(_arr != NULL)
	    delete []_arr;
	// substitute new array for old array
	_arr = arrNew;
}


#endif