/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckDialogControl.h"
#include "ckComboBoxControl.h"
#include "ckControlTable.h"
#include "ckControlImp.h"
#include "ckScanner.h"
#include "ckMiscParser.h"
#include "ckStream.h"
#include "ckVariables.h"
#include "ckValueWrapper.h"
#include "ckSettings.h"
#include "ckPainter.h"

//
// static 
//
ckControl::ControlStyleT ckComboBoxControl::_comboboxstyle[] =
{
    {"cbsautohscroll",          CBS_AUTOHSCROLL},
    {"cbsdisablenoscroll",      CBS_DISABLENOSCROLL},
    {"cbsdropdown",             CBS_DROPDOWN},
    {"cbsdropdownlist",         CBS_DROPDOWNLIST},
    {"cbslowercase",            CBS_LOWERCASE},
    {"cbsuppercase",            CBS_UPPERCASE},
    {"cbssimple",               CBS_SIMPLE},
    {"cbssort",                 CBS_SORT},
    {"cbownerdraw",             CBS_OWNERDRAWFIXED},
    {0, 0}
};



ckComboBoxControl::ckComboBoxControl()
:   ckControl(string(COMBOBOX_TYPE))
{

}

void ckComboBoxControl::createwindow(DWORD xstyle, const char *classname, const char *windowname, DWORD style,
                int x, int y, int w, int h, HWND parent, HMENU menu, HINSTANCE hinstance)
{
    ckControl::createwindow(xstyle, classname, windowname, style,
                x, y, w, h, parent, menu, hinstance);
    if(getHWND())
    {
        if(_attributeExists(SELECTTEXTCOLOR_ATTRIB))
        {
            _selfontcolor.reset(new COLORREF);
            *(_selfontcolor.get()) = strtocolor(getAttribute(SELECTTEXTCOLOR_ATTRIB).c_str());
        }

        if(_attributeExists(SELELECTCOLOR_ATTRIB))
        {
            _selcolor.reset(new COLORREF);
            *(_selcolor.get()) = strtocolor(getAttribute(SELELECTCOLOR_ATTRIB).c_str());
            _selbrush.reset(new Brush(*(_selcolor.get())));
            if(!(HGDIOBJ)(*_selbrush.get()))
                ckLogger::instance()->log(FAIL, "Failed to load select brush");
        }
    }
}

ckError ckComboBoxControl::performAction(string const &action, ckValueWrapper &wrapper) 
{
    if(getHWND() != NULL)
    {
        _LSDEBUG2("ckButtonControl::performAction() action: %s", action.c_str());
        return ckControlImpMap<ckComboBoxImp>::instance()->execute(action, wrapper, (*this));
    }
    else
    {
        _LSDEBUG1("ckComboBoxControl::performAction() called when window hasn't been created");
        return FAIL;
    }
}


ckError ckComboBoxControl::retreiveVariable(string const &var, string &value)
{ 
    string varname;
    ckIndexParser parser;
    parser.parse(ckStrStream(var));
    int indexed = parser.getIndex();
    varname = parser.getString();
    int stringpos = -1;
    value.erase();

    if(stricmp(var.c_str(), TITLE_ATTRIB) == 0 ||
       stricmp(var.c_str(), CKCOMBOBOXEDIT) == 0)
    {// using ckDialogImp here
        value = ckControlImpMap<ckDialogImp>::instance()->getImp()->_getText(*this);
    }
    //
    // .sel property
    //
    else if((stringpos = stricmp(var.c_str(), CKEDITSEL)) == 0 ||
            stricmp(varname.c_str(), CKLISTBOXSELPOS) == 0    )
    {
        int cursel = ckControlImpMap<ckComboBoxImp>::instance()->getImp()->_getCurSel(*this);
        _LSDEBUG2("ckComboBoxControl::retreiveVariable() cur sel: %d", cursel);
        string str = ckControlImpMap<ckComboBoxImp>::instance()->getImp()->_getText(*this, cursel);
        _LSDEBUG2("ckComboBoxControl::retreiveVariable() cur sel text : %s", str.c_str());
        
        if(stringpos == 0)
            value = str;
        else
        {
            char buffer[0xFF];
            value = itoa(cursel, buffer, 10);
        }

        _LSDEBUG2("ckComboBoxControl::retreiveVariable() result : %s", value.c_str());
    }
    //
    // .all property
    //
    else if(stricmp(varname.c_str(), CKLISTBOXALL) == 0)
    {
        ckComboBoxImp *imp = ckControlImpMap<ckComboBoxImp>::instance()->getImp();
        bool started = false;
                
        _LSDEBUG1("ckComboBoxControl::retreiveVariable() getting all");
        for(int i=0, size=imp->_getCount(*this); i < size; ++i)
        {
            if(indexed == -1)
            {
                if(started)
                {
                    _LSDEBUG2("ckListBoxControl::retreiveVariable() seperator: %s", ckSettings::instance()->getSeparator().c_str());
                    value += ckSettings::instance()->getSeparator();
                }
                else
                    started = true;

                value += imp->_getText(*this, i);
            }
            else if(indexed-- == 0)
            {
                //it's the right index
                value = imp->_getText(*this, i);
                break;
            }
        }
        _LSDEBUG2("ckComboBoxControl::retreiveVariable() all: %s", value.c_str());
    }
    else
    {
        _LSDEBUG1("ckComboBoxControl::retreiveVariable() getting misc");
        return ckControl::retreiveVariable(var, value);
    }
    
    return SUCCESS;
}

/*
DWORD ckComboBoxControl::_parseStyle()
{
    DWORD style = WS_VISIBLE | CBS_HASSTRINGS;

    _LSDEBUGFUNS("ckComboBoxControl::_parseStyle()");

    {
    _LSDEBUG1("ckComboBoxControl::_parseStyle() parsing button style");
    ckStyleParser styleparser(style, ckComboBoxControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }
    {
    _LSDEBUG1("ckComboBoxControl::_parseStyle() parsing dialog style");
    //cool way for data inheritance?
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }

    _LSDEBUG2("ckComboBoxControl::_parseStyle() got button style: %08X", style);
    _LSDEBUG1("WARNING: should probably do a style check if want ot be safe");
    _LSDEBUG1("ckComboBoxControl::_parseStyle() we should warn user if invalid style!");
    
    if(parentHWND())
    {
        _LSDEBUG1("ckComboBoxControl::_parseStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckComboBoxControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckComboBoxControl::_parseStyle()");

    return style;
}
*/

void ckComboBoxControl::acceptPainter(ckPainter &painter)
{
    painter.paintComboBox(*this);
}