/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_CONTROLIMP_H_
#define _CK_CONTROLIMP_H_

#include "ckDialog.h"

class ckValueWrapper;
class ckControl;

class ckControlImp
{
public:
    virtual ckError show(ckValueWrapper &wrapper, ckControl &control) 
    { 
        _LSDEBUG1("ckControlImp::show() is called");
        return NOT_SUPPORTED;
    }

    virtual ckError resize(ckValueWrapper &wrapper, ckControl &control) 
    {
        _LSDEBUG1("ckControlImp::resize() is called");
        return NOT_SUPPORTED;
    }
    virtual ckError move(ckValueWrapper &wrapper, ckControl &control)
    {
        _LSDEBUG1("ckControlImp::move() is called");
        return NOT_SUPPORTED;
    }

    virtual ckError state(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckControlImp::state() is called");
        return NOT_SUPPORTED;
    }

    virtual ckError set(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckControlImp::set() is called");
        return NOT_SUPPORTED;
    }

    virtual ckError line(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckControlImp::(() is called");
        return NOT_SUPPORTED;
    }

    virtual ckError setui(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckControlImp::setui() is called");
        return NOT_SUPPORTED;
    }
};


//
// ckControlImpMap
//
template <class T>
class ckControlImpMap
{
public:
    typedef ckError (T::*action)(ckValueWrapper &wrapper, ckControl &control);
    static ckControlImpMap *instance();
    ckError execute(string const &impname, ckValueWrapper &wrapper, ckControl &control);
    T *getImp() { return _imp.get();}

protected:

    #define MAX_TABLE_LENGTH    10
    #define MAX_ACTION_NAME     0x10

    ckControlImpMap();

    typedef struct
    {
        char _name[MAX_ACTION_NAME];
        action _func;
    } ckControlImpMapT;

    static auto_ptr<ckControlImpMap<T> > _instance;
    ckControlImpMap<T>::ckControlImpMapT _table[MAX_TABLE_LENGTH];
    //map<string, action> _action;
    auto_ptr<T> _imp;
};

//
// Global
//
template <class T>
auto_ptr<ckControlImpMap<T> > ckControlImpMap<T>::_instance;

template <class T>
ckControlImpMap<T> *ckControlImpMap<T>::instance()
{
    if(_instance.get() == NULL)
    {
        _instance.reset(new ckControlImpMap<T>);
        _LSDEBUG1("TRACING ckControlImpMap created....");
    }

    return _instance.get();
}

template <class T>
ckControlImpMap<T>::ckControlImpMap()
{
    assert(!_imp.get());
    _imp.reset(new T);
    
    struct  { 
                const char * _name; 
                action _func;
            } 
                table[] =  
                {
                    {"show",        &T::show},
                    {"resize",      &T::resize},
                    {"move",        &T::move},
                    {"state",       &T::state},
                    {"set",         &T::set},
                    {"line",        &T::line},
                    {"setui",       &T::setui},
                    {0, 0}
                };

    assert(_imp.get() != NULL);
    
    int i=0;
    while(table[i]._name) //copy the 0's too
    {
        _LSDEBUG2("i = ", i);
        _LSDEBUG2("table[i]._name = ", table[i]._name);
        _LSDEBUG2("strlen(table[i]._name)  = ", strlen(table[i]._name));

        assert( i < MAX_TABLE_LENGTH);
        assert(strlen(table[i]._name) < MAX_ACTION_NAME);
        

        strcpy(_table[i]._name, table[i]._name);
        _table[i]._func = table[i]._func;
        i++;
    }
    
    _table[i]._name[0] = '\0';
    _table[i]._func = NULL;

    _LSDEBUG0;
}

template <class T>
ckError ckControlImpMap<T>::execute(string const &impname, ckValueWrapper &wrapper, ckControl &control) 
{ 
    _LSDEBUG0;
    int i = 0;
    _LSDEBUG0;
    assert(_imp.get());
    while(_table[i]._name[0])
    {
        _LSDEBUG0;
        if(stricmp(impname.c_str(), _table[i]._name) == 0)
        {
            return ((_imp.get())->*(_table[i]._func))(wrapper, control);
        }

        ++i;
    }
    
    _LSDEBUG0;
    _LSDEBUG2("ckControlImpMap()::execute() action %s not found", impname.c_str());
    
    return FAIL;
}


//
// ckDialogImp
//
class ckDialogImp : public ckControlImp
{
public:
    ckDialogImp()
    {
        _LSDEBUG1("ckDialogImp() ctor");
    }

    virtual ~ckDialogImp() 
    {
        _LSDEBUG1("~ckDialogImp() dtor");
    }

    //
    // ckControlImp interface
    //
    virtual ckError show(ckValueWrapper &wrapper, ckControl &control);
    virtual ckError resize(ckValueWrapper &wrapper, ckControl &control);
    virtual ckError move(ckValueWrapper &wrapper, ckControl &control);

    // contains enable, focus
    virtual ckError state(ckValueWrapper &wrapper, ckControl &control);
    virtual ckError set(ckValueWrapper &wrapper, ckControl &control);
    virtual ckError line(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckDialogImp::line() is called");
        return NOT_SUPPORTED;
    }

    virtual ckError setui(ckValueWrapper &wrapper, ckControl &control);
    
    //
    // Other Manipulation functions
    //
    virtual void _setText(ckControl &control, string &str);
    virtual string _getText(ckControl &control);
    virtual int _getTextLength(ckControl &control);

protected:
    // true means processed
    virtual bool _enable(string const &str, bool status, HWND hwnd);
    virtual bool _focus(string const &str, HWND hwnd);
};



//
// ckStaticImp
//
class ckStaticImp : public ckDialogImp
{
public:
    ckStaticImp()
    {}

    virtual ~ckStaticImp() {}
    virtual ckError setui(ckValueWrapper &wrapper, ckControl &control);
};



//
// ckButtonImp
//
class ckButtonImp : public ckDialogImp
{
public:
    ckButtonImp()
    {}

    virtual ~ckButtonImp() {}

    //
    // contains enable, focus, check, push
    virtual ckError state(ckValueWrapper &wrapper, ckControl &control);
    
    virtual ckError setui(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckButtonImp::setui() is called");
        return NOT_SUPPORTED;
    }

protected:
    virtual bool _push(string const &str, bool status, HWND hwnd);
    virtual bool _check(string const &str, string const &arg, HWND hwnd);
};


//
// ckLineImp
//
class ckLineImp : public ckDialogImp
{
public:
    ckLineImp() {}
    virtual ~ckLineImp() {}

    virtual ckError line(ckValueWrapper &wrapper, ckControl &control);
    virtual ckError setui(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckControlImp::setui() is called");
        return NOT_SUPPORTED;
    }

protected:
    virtual ckError _add(int num, ckValueWrapper &wrapper, ckControl &control) = 0;
    virtual ckError _delete(int num, ckValueWrapper &wrapper, ckControl &control) = 0;
    virtual ckError _overwrite(int num, ckValueWrapper &wrapper, ckControl &control) = 0;
};

//
// ckEditImp
//
class ckEditImp : public ckLineImp
{
public:
    ckEditImp()
    {}

    virtual ~ckEditImp() {}

    //
    // ckControlImp interface
    //
    virtual ckError setui(ckValueWrapper &wrapper, ckControl &control)
    { 
        _LSDEBUG1("ckEditImp::setui() is called");
        return NOT_SUPPORTED;
    }

    // Other Manipulation functions
    void _select(ckControl &control, int start, int end);
    string _getSelect(ckControl &control);
    int _getIndex(ckControl &control, int number);
    void _selectAll(ckControl &control);
    void _clearSelect(ckControl &control);
    void _setAppend(ckControl &control);
    void _replaceSelection(ckControl &control, string const &str);
    void _append(ckControl &control, string const &str);

protected:
    virtual ckError _add(int num, ckValueWrapper &wrapper, ckControl &control);
    virtual ckError _delete(int num, ckValueWrapper &wrapper, ckControl &control);
    virtual ckError _overwrite(int num, ckValueWrapper &wrapper, ckControl &control);
};

//
// ckListBoxImp
//
class ckListBoxImp : public ckLineImp
{
public:
    ckListBoxImp() {}
    virtual ~ckListBoxImp() {}

    //
    // general
    //
    int     _getCount(ckControl &control);
    string  _getText(ckControl &control, int num);
    int     _getTextLength(ckControl &control, int num);
    smart_array<int> _getSelItems(ckControl &control, int &total);
    bool    _isSelected(ckControl &control, int num);

    //
    // single selection
    //
    int     _getCurSel(ckControl &control);
    //
    // multiselection
    //
    int     _getSelCount(ckControl &control);

protected:
    virtual ckError _add(int num, ckValueWrapper &wrapper, ckControl &control);
    virtual ckError _delete(int num, ckValueWrapper &wrapper, ckControl &control);
    virtual ckError _overwrite(int num, ckValueWrapper &wrapper, ckControl &control);
};

//
// ckComboBoxImp
//
class ckComboBoxImp : public ckLineImp
{
public:
    ckComboBoxImp() {}
    virtual ~ckComboBoxImp() {}

    int     _getCurSel(ckControl &control);
    string  _getText(ckControl &control, int num);
    int     _getTextLength(ckControl &control, int num);
    int     _getCount(ckControl &control);

protected:
    virtual ckError _add(int num, ckValueWrapper &wrapper, ckControl &control);
    virtual ckError _delete(int num, ckValueWrapper &wrapper, ckControl &control);
    virtual ckError _overwrite(int num, ckValueWrapper &wrapper, ckControl &control);
};

#endif