/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_CONTROLMANAGER_H_
#define _CK_CONTROLMANAGER_H_

#include "ckDialog.h"
#include "ckControl.h"

class ckBuilder
{
public:
    virtual ~ckBuilder() {}

    virtual void startBuilding(string const &type) = 0;
    virtual void addAttribute(string const &att, ckValueWrapper &wrapper) = 0;
    virtual void finishBuilding(string const &type) = 0;
    virtual void terminateBuilding() = 0;
};

class ckControlBuilder : public ckBuilder
{
public:
    ckControlBuilder();

    virtual auto_ptr<ckControl> get() ;

    virtual void reset();

    virtual void startBuilding(string const &type);
    virtual void addAttribute(string const &att, ckValueWrapper &wrapper);
    virtual void finishBuilding(string const &type);
    virtual void terminateBuilding();
    ckError buildingLog()
    {   return _log;    }

    bool doingProject()
    {   return _projectStarted; }

protected:
    ckError _log;
    bool _projectStarted;
    ckControl* _current;
    auto_ptr<ckControl> _top;
    auto_ptr<ckControl> _product;
};

class ckControlManager : public ckBuilder
{
public:
    static ckControlManager * instance();

    //
    // ckBuilder
    //
    virtual void startBuilding(string const &type);
    virtual void addAttribute(string const &att, ckValueWrapper &wrapper);
    virtual void finishBuilding(string const &type);
    virtual void terminateBuilding() { _builder.terminateBuilding();}
    ckError checkSyntax() { return _builder.buildingLog();}
    bool doingProject() { return _builder.doingProject();}

    
    //
    // ckControl recursive manipulation
    //
    bool createWindow(string const &id);

    // active
    // !ckAction
    ckError perform(string const &path, string &arguments);

    // template
    // *onckDialogEvent
    ckError addEventHandler(string const &eventpath, string &handler);

    // both
    ckError setVariable(string const &path, string const &value);

    // both
    ckError retreiveVariable(string const &path, string &result);

    // active
    ckError remove(ckControl &control);

protected:
    
    auto_vector<ckControl>::iterator _findChild(string const &id);

    static auto_ptr<ckControlManager> _instance;
    ckControlBuilder _builder;
    auto_vector<ckControl> _template;
    auto_vector<ckControl> _active;
};


#endif





