/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
    

#include "ckDialog.h"
#include "ckControl.h"
#include "ckControlManager.h"
#include "ckPainter.h"



LRESULT onCTLColor(ckControl *control, HDC hdc, HWND hwnd, UINT msg);

//
// global static
//
HINSTANCE ckControl::_hInstance = NULL;

bool ckControl::init(HINSTANCE hinst)
{
    WNDCLASSEX wndclass;

    _hInstance = hinst;
    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_DBLCLKS;
    wndclass.cbClsExtra = 0;
    wndclass.cbWndExtra = 0;
    wndclass.hIcon = NULL;
    wndclass.hCursor = ::LoadCursor (0, IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW);
    wndclass.lpszMenuName = NULL;
    wndclass.hIconSm = NULL;
    wndclass.hInstance = g_hInstance;
    wndclass.lpfnWndProc = WindowProc;
    wndclass.lpszClassName = CKDIALOGCLASSNAME;
    return (RegisterClassEx(&wndclass) != NULL);
}

//
// main window proc
//
LRESULT CALLBACK ckControl::WindowProc(HWND hwnd,	UINT msg,	WPARAM wParam,	LPARAM lParam)
{
    LRESULT lr = 1; //default is not processed

    ckControl *control = (ckControl *)GetWindowLong(hwnd, GWL_USERDATA);

    switch(msg)
    {

    //=========================
    // Window Messages
    // 
    case WM_CREATE:
        {
            /*
            LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
            assert(lpcs != NULL);
            assert(lpcs->lpCreateParams != NULL);
            _LSDEBUG0;
            _LSDEBUG1("WM_CREATE dumping createstruct");
            _LSDEBUG2("WM_CREATE got control class", lpcs->lpszClass);
            _LSDEBUG2("WM_CREATE got control title", lpcs->lpszName);
            _LSDEBUG2("WM_CREATE got control x", lpcs->x);
            _LSDEBUG2("WM_CREATE got control y", lpcs->y);
            _LSDEBUG2("WM_CREATE got control width", lpcs->cx);
            _LSDEBUG2("WM_CREATE got control height", lpcs->cy);
            _LSDEBUG2("WM_CREATE got control parent", (DWORD)lpcs->hwndParent);
            _LSDEBUG2("WM_CREATE got control %08X from param", (DWORD)(lpcs->lpCreateParams));
            control = reinterpret_cast<ckControl *>(lpcs->lpCreateParams);
            _LSDEBUG2("WM_CREATE got control %08X", (DWORD)control);
            //somehow sending event here causes problem
            //_LSDEBUG1("WM_CREATE sending init event");
            //control->notify(INIT_EVENT);
            _LSDEBUG0;
            SetWindowLong(hwnd, GWL_USERDATA, (LONG)control);
            _LSDEBUG0;
            */
            lr = 0;
            _LSDEBUG0;
        }
        //_LSDEBUG1("WM_CREATE sending init event");
        //control->notify(INIT_EVENT);
        _LSDEBUG0;
        break;

    case WM_ERASEBKGND:
        {
            if(control)
            {
                Brush *bgbrush = control->getBG();
                if(bgbrush)
                {
                    HDC hdc = (HDC) wParam; // handle of device context 
                    RECT rc;
                    GetClientRect(hwnd, &rc);
                    FillRect(hdc, &rc, (HBRUSH)(HGDIOBJ)*bgbrush);
                    lr = TRUE;
                }
                else
                {
                    lr = DefWindowProc(hwnd, msg, wParam, lParam);
                }
            }
        }
        break;
        
    case WM_DESTROY:
        if(control)
        {   
            ckError error;

            //the following two will be taken care in ckControl dtor
            //to make sure that they all goto the event
            //SetWindowLong(hwnd, GWL_USERDATA, 0);
            //error = control->notify(DESTROY_EVENT);
            //we ignore the result that if no handler's found
            error = ckControlManager::instance()->remove(*control);
            assert(error == SUCCESS);
            //PostQuitMessage(0);
            lr = 0;
        }
        break;

    case WM_SHOWWINDOW:
        lr = 0;
        break;
    // Control Messages
    // 

    case WM_COMMAND:

        control = (ckControl *)GetWindowLong((HWND)lParam, GWL_USERDATA);
        
        switch(HIWORD(wParam))
        {
        //
        // Button Commands
        //
        case BN_CLICKED:
            if(control)
            control->notify(CKBNCLICK_EVENT);
            break;            
        case BN_DBLCLK:
            if(control)
            control->notify(CKBNDBLCLICK_EVENT);
            break;
        case BN_DISABLE:
            if(control)
            control->notify(CKBNDISABLE_EVENT);
            break;
        case BN_PUSHED:
            if(control)
            control->notify(CKBNPUSH_EVENT);
            break;
        case BN_UNPUSHED:
            if(control)
            control->notify(CKBNUNPUSH_EVENT);
            break;
        //
        // Edit Commands
        //
        case EN_CHANGE:
            if(control)
            control->notify(CKEDCHANGE_EVENT);
            break;
        case EN_SETFOCUS:
            if(control)
            control->notify(CKEDFOCUS_EVENT);
            break;
        case EN_KILLFOCUS:
            if(control)
            control->notify(CKEDBLUR_EVENT);
            break;

        //
        // List Box Commands
        //
        /*
        case LBN_DBLCLK:
            if(control)
            control->notify(CKLBDBLCLICK_EVENT);
            break;
        case LBN_SETFOCUS:
            if(control)    
            control->notify(CKLBFOCUS_EVENT);
            break;
        case LBN_KILLFOCUS:
            if(control)            
            control->notify(CKLBBLUR_EVENT);
            break;
        case LBN_SELCANCEL:
            if(control)
            control->notify(CKLBSELCANCEL_EVENT);
            break;
        */

        case LBN_SELCHANGE:
            if(control)
            control->notify(CKLBSELCHANGE_EVENT);
            break;
        //
        // Combo Box Commands
        //
        case CBN_CLOSEUP:
            if(control)
            control->notify(CKCBCLOSEUP_EVENT);
            break;
        /*
        case CBN_DBLCLK:
            if(control)
            control->notify(CKCBDBLCLICK_EVENT);
            break;
        case CBN_EDITCHANGE:
            if(control)
            control->notify(CKCBEDITCHANGE_EVENT);
            break;
        case CBN_SETFOCUS:
            if(control)
            control->notify(CKCBFOCUS_EVENT);
            break;
        case CBN_KILLFOCUS:
            if(control)
            control->notify(CKCBBLUR_EVENT);
            break;
        case CBN_SELCHANGE:
            if(control)
            control->notify(CKCBSELCHANGE_EVENT);
            break;
        */
      
        case CBN_DROPDOWN:
            if(control)
            control->notify(CKCBDROPDOWN_EVENT);
            break;
        case CBN_SELENDCANCEL:
            if(control)
            control->notify(CKCBSELENDCANCEL_EVENT);
            break;
        case CBN_SELENDOK:
            if(control)
            control->notify(CKCBSELENDOK_EVENT);
            break;

        //
        // Static Commands
        //
        /*
        case STN_CLICKED:
            if(control)
                control->notify(CKSTCLICKED_EVENT);
            break;
        case STN_DBLCLK:
            if(control)
                control->notify(CKSTDBLCLICK_EVENT);
            break;
        case STN_DISABLE:
            if(control)
                control->notify(CKSTDISABLE_EVENT);
            break;
        case STN_ENABLE:
            if(control)
                control->notify(CKSTENABLE_EVENT);
            break;
        */
        default:
            //unhandled messages
            _LSDEBUG1("Unhandled message");
        }   

        lr = DefWindowProc(hwnd, msg, wParam, lParam); 
        break;
    
    /*
    case WM_CTLCOLORBTN:
        {
            HDC hdc = (HDC) wParam;
            HWND hwndbutton = (HWND) lParam;
            RECT r;

            SetTextColor(hdc, RGB(0xFF, 0xFF, 0xFF));
            GetClientRect(hwndbutton, &r);
            FillRect(hdc, &r, (HBRUSH)GetStockObject(LTGRAY_BRUSH));

            SelectObject(hdc, GetStockObject(LTGRAY_BRUSH));
            lr = (LRESULT) GetStockObject(LTGRAY_BRUSH);
        }
        break;
    */

    // CTL codes must be modified
    case WM_CTLCOLOREDIT:
        lr = onCTLColor((ckControl *)GetWindowLong((HWND)lParam, GWL_USERDATA), (HDC)wParam, hwnd, msg);
        break;

    case WM_CTLCOLORLISTBOX:
        lr = onCTLColor((ckControl *)GetWindowLong((HWND)lParam, GWL_USERDATA), (HDC)wParam, hwnd, msg);
        break;
        
        /*
    case WM_MEASUREITEM:
        if(control)
        {
            LPMEASUREITEMSTRUCT lpmis = (LPMEASUREITEMSTRUCT)lParam; 
            control->getFont();
            if(lpmis->itemHeight <  

        }
        break;
        */
    case WM_DRAWITEM:
        {
            control = (ckControl *)GetWindowLong(((LPDRAWITEMSTRUCT)lParam)->hwndItem, GWL_USERDATA);
			assert(control);
            ckOwnerPainter painter((LPDRAWITEMSTRUCT)lParam);
            control->acceptPainter(painter);
            //lr = DefWindowProc(hwnd, msg, wParam, lParam);
            lr = TRUE;
        }
        break;

    default:
        //=========================
        // Default fall-through Messages
        // 
        
        // Fall through messages that we notify but don't process
        //

        switch(msg)
        {
        
            case WM_CLOSE:
                _LSDEBUG2("WM_CLOSE got control %08X", (DWORD)control);
                lr = DefWindowProc(hwnd, msg, wParam, lParam);
                break;
            case WM_KILLFOCUS:
                if(control)
                control->notify(CKBLUR_EVENT);
                break;
            case WM_SETFOCUS:
                if(control)
                control->notify(CKFOCUS_EVENT);
                break;
            case WM_MOVE:
                if(control)
                control->notify(CKMOVE_EVENT);
                break;
            case WM_SIZE:
                if(control)
                control->notify(CKSIZE_EVENT);
                break;
            case WM_LBUTTONDBLCLK:
                if(control)
                control->notify(CKLBUTTONDBLCLICK_EVENT);
                break;
            case WM_LBUTTONDOWN:
                if(control)
                control->notify(CKLBUTTONDDOWN_EVENT);
                break;
            case WM_LBUTTONUP:
                if(control)
                control->notify(CKLBUTTONDUP_EVENT);
                break;
        }

        lr = DefWindowProc(hwnd, msg, wParam, lParam);
    }

    return lr;
}



LRESULT onCTLColor(ckControl *control, HDC hdc, HWND hwnd, UINT msg)
{
    LRESULT lr;

    if(control)
    {
        if(control->getBGColor())
        {
            SetBkColor(hdc, *(control->getBGColor()));
            _LSDEBUG2("WM_CTLCOLOREDIT getting bg color", (DWORD)*(control->getBGColor()));
            lr = (LRESULT)(HGDIOBJ)(*(control->getBG()));
            _LSDEBUG2("WM_CTLCOLOREDIT returning brush", (DWORD)(HGDIOBJ)(*(control->getBG())));
        }
        else
        {
            lr = DefWindowProc(hwnd, msg, (WPARAM)hdc, (LPARAM)control->getHWND());
        }

        if(control->getFontColor())
        {
            SetTextColor(hdc, *(control->getFontColor()));
            _LSDEBUG2("WM_CTLCOLOREDIT getting font color", (DWORD)*(control->getFontColor()));
        }

    }
    else
    {
        lr = DefWindowProc(hwnd, msg, (WPARAM)hdc, (LPARAM)control->getHWND());
    }

    return lr;
}