/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_DIALOGMANAGER_H_
#define _CK_DIALOGMANAGER_H_

#include "ckDialog.h"
#include "ckControl.h"
#include "ckStream.h"
#include "ckScanner.h"
#include "ckValueWrapper.h"


//this manager could be a subclass of ckControl when
//i get more time to write it
class ckDialogManager
{
public:
    static ckDialogManager *instance()
    {
        if(_instance.get() == NULL)
        {
            _instance.reset(new ckDialogManager);
        }

        assert(_instance.get() != NULL);
        return _instance.get();
    }

    virtual ~ckDialogManager() 
    {
        _LSDEBUG1("ckDialogManager() dtor");
        _LSDEBUG2("_pool size ", _pool.size());
        _pool.empty();
        _LSDEBUG2("_activepool size ", _activepool.size());
        _activepool.empty();
    }

    virtual void add(auto_ptr<ckControl> &dialog)
    {
        assert(dialog.get() != NULL && stricmp(dialog->getType().c_str(), DIALOG_TYPE) == 0);
        _pool.push_back(dialog);
    }

    //
    // all recursive functions
    //
    bool createWindow(string const &id)
    {
        /*
        _LSDEBUG2("ckDialogManager::createWindow() checking id %s", id.c_str());
        auto_vector<ckControl>::iterator it;
        for(it=_pool.begin(); it != _pool.end(); ++it)
        {
            _LSDEBUG2("ckDialogManager::createWindow() checking against id %s", (it->getAttribute(ID_ATTRIB)).c_str());
            if(stricmp(it->getAttribute(ID_ATTRIB).c_str(), id.c_str()) == 0)
            {
                _LSDEBUG1("ckDialogManager::createWindow() id found in dialog pool");
                _LSDEBUG2("ckDialogManager::createWindow() creating title", (it->getAttribute(TITLE_ATTRIB)).c_str());
                
                auto_ptr<ckControl> clone = it->Clone();
                //we should change the clone's name!!
                _LSDEBUG1("WARNING: we should change the clone's id!!");
                _LSDEBUG2("ckDialogManager::createWindow() cloned at: %08X", (DWORD)(ckControl *)clone.get());
                bool result = clone->createWindow();
                
                _LSDEBUG0;

                if(result)
                {
                    _LSDEBUG0;
                    _activepool.push_back(clone);
                }
                else
                {
                    _LSDEBUG1("ckDialogManager::createWindow() it->createWindow failed");
                }

                return result;
            }
        }
        
        _LSDEBUG1("ckDialogManager::createWindow() id not found in dialog pool");
        return false;
        */
        return true;
    }

    // 
    // perform
    //
    // only works for active dialogs
    // (tested)
    ckError perform(string const &id, string &arguments)
    {
        /*
        ckStrStream stream(id);
        ckNodePathScanner scanner(stream);
        if(scanner.accept() == ckTokenString)
        {
            auto_vector<ckControl>::iterator it = _findcontrol(scanner.getString());
            if(_isfound(it))
            {
                return it->nodeRecursivePerform(scanner, arguments, ckControl::PERFORM_ACTION);
            }
        }   
        
        _LSDEBUG2("ckDialogManager::perform() path failed", id.c_str());
        return FAIL;
        */
        return SUCCESS;
    }

    //
    // addEventHandler
    //
    // only works for dialog templates
    // (tested)
    ckError addEventHandler(string const &event, string &handler)
    {
        /*
        ckStrStream stream(event);
        ckNodePathScanner scanner(stream);
        ckValueWrapper wrapper(scanner);
        string str;

        _LSDEBUG0;
        ckError error = wrapper.getStringWithLength(str);
        if(error == SUCCESS)
        {
            _LSDEBUG0;
            auto_vector<ckControl>::iterator it = _findstockcontrol(str);

            if(_isfound(it))
            {
                _LSDEBUG0;
                return it->nodeRecursivePerform(scanner, handler, ckControl::PERFORM_ADDEVENT);
            }
        }
        
        /*
        if(scanner.accept() == ckTokenString)
        {
            _LSDEBUG0;
            auto_vector<ckControl>::iterator it = _findstockcontrol(scanner.getString());

            if(_isfound(it))
            {
                _LSDEBUG0;
                return it->nodeRecursivePerform(scanner, handler, ckControl::PERFORM_ADDEVENT);
            }
        }
        */
        
        /*
        _LSDEBUG2("ckDialogManager::addEventHanlder() path failed", event.c_str());
        return FAIL;
        */
        return SUCCESS;
    }

    //
    // setAttribute
    //
    // only work with active dialogs
    ckError setAttribute(string const &path, string &value)
    {
        /*
        ckStrStream stream(path);
        ckNodePathScanner scanner(stream);
        ckValueWrapper wrapper(scanner);
        string str;

        _LSDEBUG0;
        ckError error = wrapper.getStringWithLength(str);
        if(error == SUCCESS)
        {
            auto_vector<ckControl>::iterator it = _findcontrol(str);
            
            if(_isfound(it))
            {
                _LSDEBUG0;
                return it->nodeRecursivePerform(scanner, value, ckControl::SET_ATTRIBUTE);
            }
            
            _LSDEBUG0;
        }
    
        _LSDEBUG2("ckDialogManager::retreiveVariable() path failed", path.c_str());

        return FAIL;
        */
        return SUCCESS;
    }

    //
    // retreiveVariable
    //
    // works with both
    // (tested)
    ckError retreiveVariable(string const &path, string &result)
    {
        /*
        ckStrStream stream(path);
        ckNodePathScanner scanner(stream);
        ckValueWrapper wrapper(scanner);
        string str;

        _LSDEBUG0;
        ckError error = wrapper.getStringWithLength(str);
        if(error == SUCCESS)
        {
            auto_vector<ckControl>::iterator it = _findcontrol(str);
            if(!_isfound(it))
                it = _findstockcontrol(str);

            if(_isfound(it))
            {
                _LSDEBUG0;
                return it->nodeRecursivePerform(scanner, result, ckControl::RETREIVE_VARIABLE);
            }
            
            _LSDEBUG0;
        }
    
        _LSDEBUG2("ckDialogManager::retreiveVariable() path failed", path.c_str());

        return FAIL;
        */
        return SUCCESS;
    }

    ckError remove(ckControl &control)
    {
        /*
        ckError error;
        auto_vector<ckControl>::iterator it = _activepool.find(&control);

        if(it != _activepool.end())
        {
            _LSDEBUG2("ckDialogManager::remove() removing toplevel control with id: %s", it->getAttribute(ID_ATTRIB).c_str());
            _LSDEBUG2("before remove _activepool size ", _activepool.size());
            _activepool.remove(it); //toplevel
            _LSDEBUG2("after remove _activepool size ", _activepool.size());
            error = SUCCESS;
        }
        else
        {   // not top level
            error = FAIL;
            for(it = _activepool.begin(); it != _activepool.end(); ++it)
            {
                if(it->removeChild(control) == SUCCESS)
                {
                    //we fake the success this way, because iterator would be
                    //screwed after revmoing, or we'll need to add a error var
                    error = SUCCESS;
                    break;
                }
            }
        }

        return error;
        */
        return SUCCESS;
    }

protected:
    ckDialogManager() 
    {
        _LSDEBUG1("ckDialogManager() ctor");
    }

    static auto_ptr<ckDialogManager> _instance;

    bool _isfound(auto_vector<ckControl>::iterator const &it)
    {
        return (it != _activepool.end() && it != _pool.end());
    }

    auto_vector<ckControl>::iterator _findcontrol(string const &id)
    {
        auto_vector<ckControl>::iterator it;
        for(it = _activepool.begin(); it != _activepool.end(); ++it)
        {
            _LSDEBUG2("ckDialogManager::_findcontrol() checking id %s", id.c_str());
            _LSDEBUG2("ckDialogManager::_findcontrol() against id %s", it->getAttribute(ID_ATTRIB).c_str());
            if(stricmp(it->getAttribute(ID_ATTRIB).c_str(), id.c_str()) == 0) 
            {
                break;
            }
        }
        return it;
    }

    auto_vector<ckControl>::iterator _findstockcontrol(string const &id)
    {
        //let's check the stocked pool
        auto_vector<ckControl>::iterator it;
        for(it = _pool.begin(); it != _pool.end(); ++it)
        {
            _LSDEBUG2("ckDialogManager::_findcontrol() checking id %s", id.c_str());
            _LSDEBUG2("ckDialogManager::_findcontrol() against id %s", it->getAttribute(ID_ATTRIB).c_str());
            if(stricmp(it->getAttribute(ID_ATTRIB).c_str(), id.c_str()) == 0) 
            {
                break;
            }
        }
        return it;
    }

    auto_vector<ckControl> _activepool;
    auto_vector<ckControl> _pool;
};


#endif