/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckDialogControl.h"
#include "ckEditControl.h"
#include "ckControlTable.h"
#include "ckControlImp.h"
#include "ckScanner.h"
#include "ckMiscParser.h"
#include "ckStream.h"
#include "ckVariables.h"
#include "ckValueWrapper.h"


//
// static 
//
ckControl::ControlStyleT ckEditControl::_editstyle[] =
{
    {"esautohscroll",   ES_AUTOHSCROLL  },
    {"esautovscroll",   ES_AUTOVSCROLL  },
    {"escenter",        ES_CENTER       },
    {"esleft",          ES_LEFT         },
    {"esright",         ES_RIGHT        },
    {"eslowercase",     ES_LOWERCASE    },
    {"esuppercase",     ES_UPPERCASE    },
    {"esmultiline",     ES_MULTILINE    },
    {"eswantreturn",    ES_WANTRETURN   },
    {"esnohidesel",     ES_NOHIDESEL    },
    {"esnumber",        ES_NUMBER       },
    {"espassword",      ES_PASSWORD     },
    {"esreadonly",      ES_READONLY     },
    {0, 0}
};



ckEditControl::ckEditControl()
:   ckControl(string(EDIT_TYPE))
{

}

ckError ckEditControl::performAction(string const &action, ckValueWrapper &wrapper) 
{
    if(getHWND() != NULL)
    {
        _LSDEBUG2("ckEditControl::performAction() action: %s", action.c_str());
        return ckControlImpMap<ckEditImp>::instance()->execute(action, wrapper, (*this));
    }
    else
    {
        _LSDEBUG1("ckEditControl::performAction() called when window hasn't been created");
        return FAIL;
    }
}

ckError ckEditControl::retreiveVariable(string const &var, string &value)
{ 
    if(stricmp(var.c_str(), CKEDITTEXT)     == 0 ||
       stricmp(var.c_str(), TITLE_ATTRIB)   == 0   )
    {
        //get the text of edit
        value = ckControlImpMap<ckEditImp>::instance()->getImp()->_getText(*this);
    }
    else if(stricmp(var.c_str(), CKEDITSEL) == 0)
    {
        value = ckControlImpMap<ckEditImp>::instance()->getImp()->_getSelect(*this);
    }
    else
    {
        return  ckControl::retreiveVariable(var, value);
    }
    
    return SUCCESS;
}

/*
DWORD ckEditControl::_parseStyle()
{
    DWORD style = WS_VISIBLE;

    _LSDEBUGFUNS("ckEditControl::_parseStyle()");

    {
    _LSDEBUG1("ckEditControl::_parseStyle() parsing button style");
    ckStyleParser styleparser(style, ckEditControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }
    {
    _LSDEBUG1("ckEditControl::_parseStyle() parsing dialog style");
    //cool way for data inheritance?
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }

    _LSDEBUG2("ckEditControl::_parseStyle() got button style: %08X", style);
    _LSDEBUG1("WARNING: should probably do a style check if want ot be safe");
    _LSDEBUG1("ckEditControl::_parseStyle() we should warn user if invalid style!");
    
    if(parentHWND())
    {
        _LSDEBUG1("ckEditControl::_parseStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckEditControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckEditControl::_parseStyle()");

    return style;
}
*/