/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_EXCEPTION_H_
#define _CK_EXCEPTION_H_

#include "ckDialog.h"
#include "ckError.h"
#include "ckToken.h"

class ckException
{
public:
    ckException(ckError error, string description, ckToken token = ckTokenBeginStream)
    :   _error(error),
        _errorMsg(description),
        _token(token)
    {
    }

    ckException(ckException const &ckex)
    :   _error(ckex._error),
        _errorMsg(ckex._errorMsg),
        _token(ckex._token)
    {
    }

    ckError getError() const 
    {   return _error;      }

    string getErrorString() const
    {
        string temp;
        switch(_error)
        {
        case SUCCESS:
            temp = "SUCCESS";
            break;
        case FAIL:
            temp = "FAIL";
            break;
        case NEW_INSIDE_NEW:
            temp = "NEW_INSIDE_NEW";
            break;
        case NEW_MISMATCHED:
            temp = "NEW_MISMATCHED";
            break;
        case NEWCONTROL_MISMATCHED:
            temp = "NEWCONTROL_MISMATCHED";
            break;
        case ATTRIBUTE_OUTSIDE_NEW:
            temp = "ATTRIBUTE_OUTSIDE_NEW";
            break;
        case ATTRIBUTE_VALUE_INVALID:
            temp = "ATTRIBUTE_VALUE_INVALID";
            break;
        case INVALID_ATTRIBUTE:
            temp = "INVALID_ATTRIBUTE";
            break;
        case SYNTAX_ERROR:	
            temp = "SYNTAX_ERROR";
            break;
        case INVALID_ACTION:
            temp = "INVALID_ACTION";
            break;
        case NOT_SUPPORTED:
            temp = "NOT_SUPPORTED";
            break;
        case INVALID_VALUE:
            temp = "INVALID_VALUE";
            break;
        case INVALID_ARGUMENT:
            temp = "INVALID_ARGUMENT";
            break;
        case DUPLICATE_DEFINITION:
            temp = "DUPLICATE_DEFINITION";
            break;
        case VARIABLE_MISMATCHED:
            temp = "VARIABLE_MISMATCHED";
            break;
        default:
            assert(false);
        }

    

        return temp;
    }

    string getErrMsg() const
    {   return _errorMsg;  }

    ckToken getToken() const
    {   return _token;      }

    string getTokenString() const
    {
        string temp;
        switch(_token)
        {
        case ckTokenBeginStream: //used for debug purpose
            temp = "ckTokenBeginStream";
            break;
        case ckTokenNew: 
            temp = "ckTokenNew";
            break;
        case ckTokenInt: 
            temp = "ckTokenInt";
            break;
        case ckTokenString:
            temp = "ckTokenString";
            break;
        /*
        case ckTokenAttribute:
            temp = "ckTokenAttribute";
            break;
        */
        case ckTokenNEWControl:
            temp = "ckTokenNEWControl";
            break;
        case ckTokenENDControl:	
            temp = "ckTokenENDControl";
            break;
        case ckTokenENDNew:					
            temp = "ckTokenENDNew";
            break;
        case ckTokenBuiltinID:
            temp = "ckTokenBuiltinID";
            break;
        case ckTokenEndStream:
            temp = "ckTokenEndStream";
            break;
        case ckTokenBangString:
            temp = "ckTokenBangString";
            break;
        default:
            assert(false);
        }

        return temp;
    }
    
protected:
    const ckError _error;
    const string  _errorMsg;
    const ckToken _token;

};

#endif