/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckDialogControl.h"
#include "ckListBoxControl.h"
#include "ckControlTable.h"
#include "ckControlImp.h"
#include "ckScanner.h"
#include "ckMiscParser.h"
#include "ckStream.h"
#include "ckVariables.h"
#include "ckValueWrapper.h"
#include "ckSettings.h"

ckControl::ControlStyleT ckListBoxControl::_listboxstyle[] =
{
    {"lbsdisablenoscroll"		, LBS_DISABLENOSCROLL},
    {"lbsextendedsel"			, LBS_EXTENDEDSEL},
    {"lbsmultiplesel"           , LBS_MULTIPLESEL},
    {"lbssort"                  , LBS_SORT},
    {"lbsstandard"		        , LBS_STANDARD},
    {"lbshasstrings"            , LBS_HASSTRINGS},
    //<- must speicfy if ownerdrawn
    {"lbsnotify"                , LBS_NOTIFY},
    //<- this has to be set if only strings!!
    {"lbswantkeyboardinput"     , LBS_WANTKEYBOARDINPUT}

};


ckListBoxControl::ckListBoxControl()
:   ckControl(string(LISTBOX_TYPE))
{

}

ckError ckListBoxControl::performAction(string const &action, ckValueWrapper &wrapper) 
{
    if(getHWND() != NULL)
    {
        _LSDEBUG2("ckButtonControl::performAction() action: %s", action.c_str());
        return ckControlImpMap<ckListBoxImp>::instance()->execute(action, wrapper, (*this));
    }
    else
    {
        _LSDEBUG1("ckListBoxControl::performAction() called when window hasn't been created");
        return FAIL;
    }
}

ckError ckListBoxControl::retreiveVariable(string const &var, string &value)
{ 
    string varname;
    ckIndexParser parser;
    parser.parse(ckStrStream(var));
    int indexed = parser.getIndex();
    varname = parser.getString();
    int stringpos = -1;

    if(stricmp(var.c_str(), TITLE_ATTRIB) == 0)
    {// using ckDialogImp here
        value = ckControlImpMap<ckDialogImp>::instance()->getImp()->_getText(*this);
    }
    //
    // .sel property
    //
    else if((stringpos = stricmp(varname.c_str(), CKEDITSEL)) == 0 ||
            stricmp(varname.c_str(), CKLISTBOXSELPOS) == 0    )
    {
        int size;
        value.erase();
        bool started = false;
        ckListBoxImp *imp = ckControlImpMap<ckListBoxImp>::instance()->getImp();
        smart_array<int> selitems(imp->_getSelItems(*this, size));
        value.erase();
        _LSDEBUG2("ckListBoxControl::retreiveVariable() total selections: %d", size);

        for(int i=0; i<size; ++i)
        {
            if(imp->_isSelected(*this, selitems[i]))
            {
                if(indexed == -1)
                {
                    if(started)
                    {
                        _LSDEBUG2("ckListBoxControl::retreiveVariable() seperator: %s", ckSettings::instance()->getSeparator().c_str());
                        value += ckSettings::instance()->getSeparator();
                    }
                    else
                        started = true;

                    if(stringpos == 0)
                        value += imp->_getText(*this, selitems[i]);
                    else
                    {
                        char buffer[0xFF];
                        value += itoa(selitems[i], buffer, 10);
                    }
                }
                else if(indexed-- == 0)
                {
                    //it's the right index
                    if(stringpos == 0)
                        value = imp->_getText(*this, selitems[i]);
                    else
                    {
                        char buffer[0xFF];
                        value += itoa(selitems[i], buffer, 10);
                    }
                    break;
                }
            }
        }

        _LSDEBUG2("ckListBoxControl::retreiveVariable() result: %s", value.c_str());
    }
    //
    // .all property
    //
    else if(stricmp(varname.c_str(), CKLISTBOXALL) == 0)
    {
        ckListBoxImp *imp = ckControlImpMap<ckListBoxImp>::instance()->getImp();
        bool started = false;
        value.erase();
        
        _LSDEBUG1("ckListBoxControl::retreiveVariable() getting all");
        for(int i=0, size=imp->_getCount(*this); i < size; ++i)
        {
            if(indexed == -1)
            {
                if(started)
                {
                    _LSDEBUG2("ckListBoxControl::retreiveVariable() seperator: %s", ckSettings::instance()->getSeparator().c_str());
                    value += ckSettings::instance()->getSeparator();
                }
                else
                    started = true;

                value += imp->_getText(*this, i);
            }
            else if(indexed-- == 0)
            {
                //it's the right index
                value = imp->_getText(*this, i);
                break;
            }
        }
        _LSDEBUG2("ckListBoxControl::retreiveVariable() all: %s", value.c_str());
    }
    else
    {
        _LSDEBUG1("ckListBoxControl::retreiveVariable() getting misc");
        return ckControl::retreiveVariable(var, value);
    }
    
    return SUCCESS;
}

/*
DWORD ckListBoxControl::_parseStyle()
{
    DWORD style = WS_VISIBLE | LBS_HASSTRINGS | WS_VSCROLL;

    _LSDEBUGFUNS("ckButtonControl::_parseStyle()");

    {
    _LSDEBUG1("ckButtonControl::_parseStyle() parsing button style");
    ckStyleParser styleparser(style, ckListBoxControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }
    {
    _LSDEBUG1("ckButtonControl::_parseStyle() parsing dialog style");
    //cool way for data inheritance?
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }

    _LSDEBUG2("ckButtonControl::_parseStyle() got button style: %08X", style);
    _LSDEBUG1("WARNING: should probably do a style check if want ot be safe");
    _LSDEBUG1("ckButtonControl::_parseStyle() we should warn user if invalid style!");
    
    if(parentHWND())
    {
        _LSDEBUG1("ckButtonControl::_parseStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckButtonControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckButtonControl::_parseStyle()");

    return style;
}
*/