/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckControlParser.h"
#include "ckScanner.h"
#include "ckControlManager.h"
#include "ckValueWrapper.h"
#include "ckException.h"

ckDialogParser::ckDialogParser(ckControlManager &manager)
:   ckControlParser(manager)

{
    _LSDEBUG1("ckDialogParser::ckDialogParser() ctor");
}

ckDialogParser::~ckDialogParser()
{
	_LSDEBUG1("ckDialogParser::ckDialogParser() dtor");
}
	
ckError ckDialogParser::parse(ckStream &ckstream)
{
	ckDialogTemplateScanner scanner(ckstream);
	ckError result = SUCCESS;
	ckToken token;
	
    _LSDEBUG1("ckDialogParser::parse() entering");

	while( (token = scanner.accept()) != ckTokenEndStream)
	{
		switch(token)
		{
			case ckTokenNew:
                _LSDEBUG1("ckDialogParser::Parse() ckTokenNew");
                _LSDEBUG0;
                if(!_manager.doingProject())
                {
                    _LSDEBUG0;
                    _manager.startBuilding(DIALOG_TYPE);
                    _LSDEBUG0;
                    result = _manager.checkSyntax();
                }
                else
                {
                    _LSDEBUG0;
                    result = NEW_INSIDE_NEW;
                }
                _LSDEBUG0;
				break;

			case ckTokenENDNew:
                _LSDEBUG1("ckDialogParser::Parse() ckTokenENDNew");
                _manager.finishBuilding(DIALOG_TYPE);
                _LSDEBUG0;
                result = _manager.checkSyntax();
                _LSDEBUG0;
                if(result == NEWCONTROL_MISMATCHED || (result == SUCCESS && _manager.doingProject()))
                {
                    _LSDEBUG0;
                    result = NEW_MISMATCHED;
                }
                _LSDEBUG0;
				break;
			    
			case ckTokenNEWControl:
			{
                _LSDEBUG1("ckDialogParser::Parse() ckTokenNEWControl");
                // dialog parser is strict so can't have a top level control
                _manager.startBuilding(scanner.getString());
                _LSDEBUG0;
                result = _manager.checkSyntax();
    		}
			break;

			case ckTokenENDControl:
			{
                _LSDEBUG1("ckDialogParser::Parse() ckTokenENDControl");
                _manager.finishBuilding(scanner.getString());
                _LSDEBUG0;
                result = _manager.checkSyntax();
			}
			break;

            case ckTokenString:
                _LSDEBUG1("ckDialogParser::Parse() ckTokenString");
                _manager.addAttribute(scanner.getString(), ckValueWrapper(scanner));
                _LSDEBUG0;
                result = _manager.checkSyntax();
                break;

			default:
                _LSDEBUG1("ckDialogParser::Parse() SYNTAX_ERROR");
				result = SYNTAX_ERROR;
		}
		
		if(result != SUCCESS)
        {
            _LSDEBUG1("ckDialogParser::Parse() failed in while loop");
        }
	}
	
    if(_manager.doingProject())
    {//he's still doing project!!
        _LSDEBUG0;
        _manager.terminateBuilding();
        _LSDEBUG0;
        result = _manager.checkSyntax();
        /*
        if(result != SUCCESS)
            ckLogger::instance()->log(result, scanner.getString());
            */
        _LSDEBUG0;
    }

    _LSDEBUG2("ckDialogParser::parse() returning error code: %08X", result);

    return result;
}



//
// ckEventHandlerParser
//
/*

  class ckEventHandlerParser : public ckControlParser
public:
    ckEventHandlerParser(ckControlManager &manager);
    virtual ~ckEventHandlerParser() {}

    virtual ckError parse(ckStream &ckstream);
};
*/

ckEventHandlerParser::ckEventHandlerParser(ckControlManager &dialogmanager)
:   ckControlParser(dialogmanager)
{
}

ckError ckEventHandlerParser::parse(ckStream &ckstream)
{
    ckDialogTemplateScanner scanner(ckstream);
    string key, value;
    ckError error = SUCCESS;
    ckToken token;

    while((token = scanner.accept()) != ckTokenEndStream)
    {
        _LSDEBUG0;
        if(token == ckTokenString)
        {
            _LSDEBUG0;
            key = scanner.getString();
            _LSDEBUG0;
            ckValueWrapper wrapper(scanner);
            _LSDEBUG0;
            error = wrapper.getRawStringWithLength(value);
            _LSDEBUG0;
            if(error == SUCCESS)
            {
                _LSDEBUG2("ckEventHandlerParser::parse() adding event", key.c_str());
                _LSDEBUG2("ckEventHandlerParser::parse() adding handler", value.c_str());
                error = ckControlManager::instance()->addEventHandler(key, value);
                if(error != SUCCESS)
                {
                    _LSDEBUG1("ckEventHandlerParser::parse() failed to add event handler");
                }
            }
            else
            {
                _LSDEBUG1("ckEventHandlerParser::parse() failed to parse the value");
            }
        }
        else
        {
            error = INVALID_VALUE;
        }

        if(error != SUCCESS)
            break;
    }
    
    return error;
}
