/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckToken.h"
#include "ckScanner.h"
#include "ckStream.h"
#include "ckControlTable.h"


//
// EscapeSequence
//
const string EscapeSequence::_sequence("trn");
const string EscapeSequence::_seqmap("\t\r\n");

char EscapeSequence::getSequence(char ch)
{
    assert(_sequence.length() == _seqmap.length());
    int pos = _sequence.find(ch);
    if(pos != string::npos)
        return _seqmap[pos];

    return ch;
}

//
// ckBaseScanner
//
ckBaseScanner::ckBaseScanner(ckStream &stream, string const &separators)
:   _stream(stream),
    _token(ckTokenBeginStream),
    _separators(separators)
{
}
	
inline ckToken ckBaseScanner::token() const
{
    return _token;
}


//ckToken ckBaseScanner::peek(string separators /* = "" */)
/*
{
    if(separators.length() == 0) separators = _separators;

    return ckTokenError;
}
*/

ckToken ckBaseScanner::accept(string separators /* = "" */)
{
    if(separators.length() == 0) separators = _separators;

    if(_stream.isMore())
    {
        bool isnotSeperated = true;
        bool isQuoted = false;
        char quotech;
        char ch;
        string temp;

        do
        {
            ch = _stream.next();
            if(ch == '\\')
            {//escape
                if(_stream.isMore())
                {
                    // we should process escape stuff here such as \n \r \t
                    temp += EscapeSequence::getSequence(_stream.next());
                }
            }
            else if(isQuoted)
            {//quoted waiting for end quote
                if(ch == quotech) break;

                temp += ch;
            }
            else
            {
                isnotSeperated = (separators.find(ch) == string::npos);
                if(isnotSeperated)
                {//it's not a separator
                    if(temp.length() == 0 && (ch == '\"' || ch == '\'' || ch == '['))
                    {
                        isQuoted = true;
                        quotech = (ch != '[') ? ch : ']';
                    }
                    else
                    {
                        temp += ch;
                    }
                }
                else
                {
                    isnotSeperated = (temp.length() == 0);
                }
            }
        }
        while(_stream.isMore() && isnotSeperated);

        _LSDEBUG2("ckScanner::accept() got string \"%s\"", temp.c_str());

        _currentString = temp;
        determineToken();

        if(_currentString.length() == 0 && isQuoted)
        {
            _token = ckTokenEmptyString;
        }

        if(token() == ckTokenError && !_stream.isMore())
        {
            _token = ckTokenEndStream;
        }
    }
    else
    {
        _token = ckTokenEndStream;
    }

    _LSDEBUG2("ckScanner::accept() returning token %d", token());
    return token();
}

// make sure to return lowercase if case incensitive
string ckBaseScanner::getString()
{
    return _currentString;
}

int ckBaseScanner::getInt()
{
    return atoi(_currentString.c_str());
}

/*
string ckBaseScanner::getNextString()
{
    _LSDEBUG1("ckScanner::getNextString() not implemented yet");
    assert(false);
    return "";
}

int ckBaseScanner::getNextInt()
{
    _LSDEBUG1("ckScanner::getNextInt() not implemented yet");
    assert(false);
    return 0;
}
*/


//
// ckScanner
//
ckScanner::ckScanner(ckStream &stream, string const &separators)
:   ckBaseScanner(stream, separators)
{

}

void ckScanner::determineToken()
{
    if(_currentString.length() == 0)
    {
        _token = ckTokenError;
    }
    else if(stricmp(_currentString.c_str(), NEW_SYMBOL) == 0)
    {
        _token = ckTokenNew;
    }
    else if(stricmp(_currentString.c_str(), ENDNEW_SYMBOL) == 0)
    {
        _token = ckTokenENDNew;
    }
    else if(ckControlTable::instance()->findControl(_currentString))
    {   
        _token = ckTokenNEWControl;
    }
    else if(_currentString[0] == '~' && ckControlTable::instance()->findControl(_currentString.substr(1)))
    {//we are sure it's at least one byte long
        _token = ckTokenENDControl;
        _currentString = _currentString.substr(1);
    }
    /*
    else if(ckControlTable::instance()->findAttrib(_currentString))
    {
        _token = ckTokenAttribute;
    }
    */
    else if(false)
    {
        //we need a built-in id set
    }
    else if(isdigit(_currentString[0]) || ((_currentString[0] == '-') && (_currentString.length() > 1) && isdigit(_currentString[1])))
    {
        _token = ckTokenInt;
    }
    else if(isalpha(_currentString[0]) || _currentString[0] == '_')
    {
        _token = ckTokenString;
    }
    else if(_currentString[0] == '!')
    {
        _token = ckTokenBangString;
    }
    else if(_currentString.length() > 3 &&
            _currentString[0] == '%' && 
            _currentString[1] == '[' && 
            _currentString[_currentString.length()-1] == ']')
    {   //need to be at least 3 bytes long to have some meaning
        //right now we only support single level of variable
        _token = ckTokenVariable;
        _LSDEBUG1("ckScanner::determineToken() we don't strip variables for now");
        //_currentString = _currentString.substr(2, _currentString.length() - 3);
    }
    else
    {
        _token = ckTokenError;
    }
}


//
// ckDialogTemplateScanner
//
ckDialogTemplateScanner::ckDialogTemplateScanner(ckStream &stream)
:   ckScanner(stream, " \t\r\n")
{
    _LSDEBUG1("ckDialogTemplateScanner::ckDialogTemplateScanner() created");
}

//
// ckNodePathScanner
//
ckNodePathScanner::ckNodePathScanner(ckStream &stream)
:   ckScanner(stream, " \t\r\n.")
{

}


//
// ckStyleScanner
//
ckStyleScanner::ckStyleScanner(ckStream &stream)
:   ckScanner(stream, " \t\r\n|")
{
    

}

/*
//
// ckWholeThingScanner
//
ckWholeThingScanner::ckWholeThingScanner(ckStream &stream)
:    ckBaseScanner(stream, " \t\r\n")
{

}

// make sure to return lowercase if case incensitive
ckToken ckWholeThingScanner::accept(string separators)
{
    if(separators.length() == 0) separators = _separators;

    if(_accepted)
    {
        _token = ckTokenEndStream;
        return token();
    }

    //fill the _currentString by sucking the stream dry
    while(_stream.isMore())
    {//skip those in head
        if(_separators.find(_stream.next()) == string::npos)
            break;
    }

    _currentString = _stream.toString();

    if(_currentString.length() > 0)
    {//use a reverse stream to strip the separators
        ckReverseStrStream rstream(_currentString);

        while(rstream.isMore())
        {
            if(_separators.find(_stream.next()) == string::npos)
                break;
        }

        _currentString = rstream.toString();
    }

    _LSDEBUG2("ckWholeThingScanner::ckWholeThingScanner() ctor stripped to string %s", _currentString.c_str());
    _token  = (_currentString.length() > 0) ? ckTokenString : ckTokenEmptyString;

    return token();
}
*/