/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckScanner.h"
#include "ckValueWrapper.h"

//
// ckValueWrapper
//
ckValueWrapper::ckValueWrapper(ckBaseScanner &scanner)
:   _scanner(scanner)
{
}

string ckValueWrapper::getCurrentString()
{
    //doesn't advance or anything, just forwarding call
    return _scanner.getString();
}

ckError ckValueWrapper::getRawStringWithLength(string &str)
{
    str.erase();
    _scanner.accept();
    if(_scanner.token() != ckTokenEndStream && _scanner.token() != ckTokenEmptyString)
    {
        str = _scanner.getString();
        return SUCCESS;
    }
    
    return FAIL;
}

ckError ckValueWrapper::getRawString(string &str)
{
    str.erase();
    if(_scanner.accept() != ckTokenEndStream)
    {
        str = _scanner.getString();
        return SUCCESS;
    }
        
    return FAIL;
}

ckError ckValueWrapper::getStringWithLength(string &str, bool var /*= true */)
{
    str.erase();
    _scanner.accept();
    if(_scanner.token() == ckTokenString/* || _scanner.token() == ckTokenAttribute*/)
    {
        str = _scanner.getString();
        return SUCCESS;
    }

    return INVALID_VALUE;
}

ckError ckValueWrapper::getString(string &str, bool var /*= true*/)
{
    str.erase();
    _scanner.accept();
    if(_scanner.token() == ckTokenString || _scanner.token() == ckTokenEmptyString/* || _scanner.token() == ckTokenAttribute*/)
    {
        str = _scanner.getString();
        return SUCCESS;
    }
    return INVALID_VALUE;
}


ckError ckValueWrapper::getInt(int &num)
{
    num = -1;
    if(_scanner.accept() == ckTokenInt)
    {
        num = _scanner.getInt();
        return SUCCESS;
    }
    
    return INVALID_VALUE;
}

ckError ckValueWrapper::getInt(long &num)
{
    num = -1;
    if(_scanner.accept() == ckTokenInt)
    {
        num = (long)_scanner.getInt();
        return SUCCESS;
    }
    
    return INVALID_VALUE;
}

//
// getVariableInt
//
// post: str = integer in string form or variable
//
ckError ckValueWrapper::getVariableInt(int &num, string &str)
{
    ckError error = getInt(num);
    if(error != SUCCESS)
    {
        if(_scanner.token() == ckTokenVariable)
        {
            //we leave the num = -1
            str = _scanner.getString();
            error = SUCCESS;
        }
        else
        {
            error = INVALID_VALUE;
        }
    }
    else
    {
        char buffer[0xFF]; //255 digits, should be enough
        sprintf(buffer, "%d", num);
        str = buffer;
        
        error = SUCCESS;
    }

    return error;
}

//
// getVariableInt
//
// post: str = integer in string form or variable
//
ckError ckValueWrapper::getVariableInt(long &num, string &str)
{
    int tempnum;
    ckError error = getVariableInt(tempnum, str);
    num = (long)tempnum;
    return error;
}

ckBaseScanner &ckValueWrapper::getScanner()
{
    return _scanner;
}